:
# NAME:
#	debug.sh - debug scripts
#
# SYNOPSIS:
#	. debug.sh
#	DebugOn "e" ...
#	DebugOff "e" ...
#	DebugShell "e" ...
#	Debug "e" ...
#
#	$DEBUG_SKIP echo skipped when Debug "e" is true.
#	$DEBUG_DO echo only done when Debug "e" is true.
#	
# DESCRIPTION:
#	DebugOn turns tracing on if any "e" is found in "DEBUG_SH".
#	It turns tracing off if "!e" is found in "DEBUG_SH".
#	It also sets "DEBUG_ON" to the "e" that caused tracing to be
#	enabled, or "DEBUG_OFF" if we matched "!e".
#
#	DebugOff turns tracing on if any "e" matches "DEBUG_OFF" or
#	off if any "e" matches "DEBUG_ON". This allows nested
#	functions to not interfere with each other. 
#
#	DebugShell runs an interactive shell if any "e" is found in
#	"DEBUG_INTERACTIVE", and there is a tty available.
#	The shell used is defined by "DEBUG_SHELL" or "SHELL" and
#	defaults to '/bin/sh'.
#
#	Debug calls DebugOn and if that does not turn tracing on, it
#	calls DebugOff to turn it off.
#
#	The variables "DEBUG_SKIP" and "DEBUG_DO" are set so as to
#	enable/disable code that should be skipped/run when debugging
#	is turned on. "DEBUGGING" is the same as "DEBUG_SKIP" for
#	backwards compatability and is only set by Debug.
#
# BUGS:
#	Does not work with ksh.  If a function turns tracing on,
#	ksh turns it off when the function returns - useless.
#	
# AUTHOR:
#	Simon J. Gerraty <sjg@crufty.net>

# RCSid:
#	$Id: debug.sh,v 1.18 2009/08/24 16:32:29 sjg Exp $
#
#	@(#) Copyright (c) 1994 Simon J. Gerraty
#
#	This file is provided in the hope that it will
#	be of use.  There is absolutely NO WARRANTY.
#	Permission to copy, redistribute or otherwise
#	use this file is hereby granted provided that 
#	the above copyright notice and this notice are
#	left intact. 
#      
#	Please send copies of changes and bug-fixes to:
#	sjg@crufty.net
#

_DEBUG_SH=:

Myname=${Myname:-`basename $0 .sh`}

DEBUGGING=
DEBUG_DO=:
DEBUG_SKIP=
export DEBUGGING DEBUG_DO DEBUG_SKIP

_debugOn() {
	DEBUG_OFF=
	DEBUG_DO=
	DEBUG_SKIP=:
	DEBUG_X=-x
	set -x
	DEBUG_ON=$1
}

_debugOff() {
	DEBUG_OFF=$1
	set +x
	DEBUG_ON=$2
	DEBUG_DO=:
	DEBUG_SKIP=
	DEBUG_X=
}

# Turn on debugging if appropriate
DebugOn() {
	case "${DEBUG_SH:-$DEBUG}" in
	"")	return 1;;
	esac
	# if debuggin is off because of a !e
	# don't add 'all' to the On list.
	case "$DEBUG_OFF" in
	"")	_e=all;;
	*)	_e=;;
	esac
	for _e in ${*:-$Myname} $_e
	do
		case ",${DEBUG_SH:-$DEBUG}," in
		*,!$_e,*|*,!$Myname:$_e,*)
			# only turn it off if it was on
			$DEBUG_DO _debugOff $_e $DEBUG_ON
			return 0
			;;
		*,$_e,*|*,$Myname:$_e,*)
			# only turn it on if it was off
			$DEBUG_SKIP _debugOn $_e
			return 0
			;;
		esac
	done
	return 1
}

# Only turn debugging off if one of our args was the reason it
# was turned on.
DebugOff() {
	for _e in $*
	do
		case "$DEBUG_OFF" in
		"")	break;;
		$_e)	_debugOn $DEBUG_ON; return 0;;
		esac
	done
	for _e in $*
	do
		case "$DEBUG_ON" in
		"")	break;;
		$_e)	_debugOff; return 0;;
		esac
	done
	return 0		# always happy
}

_TTY=${_TTY:-`test -t 0 && tty`}; export _TTY

# override this if you like
_debugShell() {
	{
		echo DebugShell "$@"
		echo "Type 'exit' to continue..."
	} > $_TTY
	${DEBUG_SHELL:-${SHELL:-/bin/sh}} < $_TTY > $_TTY 2>&1
}

# Run an interactive shell if appropriate
# Note: you can use $DEBUG_SKIP DebugShell ... to skip unless debugOn
DebugShell() {
	case "$_TTY^${DEBUG_INTERACTIVE}" in
	*^|^*) return 0;;	# no tty or no spec
	esac
	for _e in ${*:-$Myname} all
	do
		case ",${DEBUG_INTERACTIVE}," in
		*,!$_e,*|*,!$Myname:$_e,*)
			return 0
			;;
		*,$_e,*|*,$Myname:$_e,*)
			# Provide clues as to why/where
			_debugShell "$_e: $@"
			return $?
			;;
		esac
	done
	return 0
}

# For backwards compatability
Debug() {
	case "${DEBUG_SH:-$DEBUG}" in
	"")	;;
	*)	DEBUG_ON=${DEBUG_ON:-_Debug}
		DebugOn $* || DebugOff $DEBUG_LAST
		DEBUGGING=$DEBUG_SKIP
		;;
	esac
}
